##
##  three simulation examples in Zhu and Zeng (2006)
##

rm(list=ls())
date()

################
################

iht <- function(y, x)
{
   x <- as.matrix(x)
   y <- scale(y)

   n <- nrow(x)
   p <- ncol(x)

   x.e <- eigen(cov(x), symmetric=T)
   halfinv <- x.e$vectors %*% diag(1/sqrt(x.e$values)) %*% t(x.e$vectors)
   z <- scale(x, center=T, scale=F) %*% halfinv

   M <- M.dimreduc.iht(y, z) 

   M.raw <- eigen(M, symmetric = T)
   M.evectors <- halfinv %*% M.raw$vectors
   M.evectors <- apply(M.evectors, 2, function(x) x/sqrt(sum(x^2)))

   list(evalues=M.raw$values, evectors=M.evectors, M=M)
}

M.dimreduc.iht <- function(y, x)
{
   p <- ncol(x)
   n <- nrow(x)

   yxmatrix <- matrix(y, ncol=p, nrow=n) * x
   Sigmayzz <- t(yxmatrix) %*% x / n;
   Betayz <- matrix(apply(yxmatrix, 2, mean), ncol=1)

   B <- matrix(0, ncol=p, nrow=p)
   B[ , 1] <- Betayz / sqrt(sum(Betayz * Betayz))

   for(i in 2:p)
   {
      B[ , i] = Sigmayzz %*% B[ , i-1]
      B[ , i] = B[ , i] / sqrt(sum(B[ , i] * B[ , i]))
   }

   B %*% t(B)
}

##########
##########

source("FourierMethod.R")
library("dr")
dyn.load("fourier-clib.dll")  
  ## or dyn.load("fourier-clib.so") in unix

n = 500; p = 10; iter = 500

b1 = matrix(c(rep(1, 4), rep(0, 6)), ncol=1)
b2 = matrix(c(rep(0, 6), rep(1, 4)), ncol=1)
BB = cbind(b1, b2)

## example 1

er1 = matrix(0, ncol=7, nrow=iter)

for(ii in 1:iter)
{
   x = matrix(rnorm(n*p), ncol=p, nrow=n)
   e = matrix(rnorm(n), ncol=1)
   y = (x %*% b1)^2 / (3 + (x %*% b2 + 2)^2) + 0.2 * e

   xy.dr = FCN(y, x, 0.1, 1.0)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 1] = dist.vb$r

   xy.dr = FMN(y, x, 0.1)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 2] = dist.vb$r

   xy.dr = dr(y~x, method="sir", nslices=5)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 3] = dist.vb$r

   xy.dr = dr(y~x, method="save", nslices=5)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 4] = dist.vb$r

   xy.dr = iht(y, x)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 5] = dist.vb$r

   xy.dr = dr(y~x, method="phdy")
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 6] = dist.vb$r

   xy.dr = dr(y~x, method="phdres")
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er1[ii, 7] = dist.vb$r
}


## example 2

er2 = matrix(0, ncol=3, nrow=iter)

for(ii in 1:iter)
{
   x = matrix(rnorm(n*p), ncol=p, nrow=n)
   e = matrix(rnorm(n), ncol=1)
   y = (x %*% b1) + 4 * (x %*% b2) * e

   xy.dr = FCN(y, x, 0.1, 1.0)
   v.est = xy.dr$evector[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er2[ii, 1] = dist.vb$r

   xy.dr = dr(y~x, method="sir", nslices=5)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er2[ii, 2] = dist.vb$r

   xy.dr = dr(y~x, method="save", nslices=5)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er2[ii, 3] = dist.vb$r
}

## example 3

er3 = matrix(0, ncol=3, nrow=iter)

for(ii in 1:iter)
{
   x = matrix(rnorm(n*p), ncol=p, nrow=n)
   e = matrix(rnorm(n), ncol=1)
   y =  (x %*% b1 + 0.2 * e > 1) + 2 * (x %*% b2 + 0.2 * e > 0)

   xy.dr = FCN(y, x, 0.1, 3.0)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er3[ii, 1] = dist.vb$r

   xy.dr = dr(y~x, method="sir", nslices=5)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er3[ii, 2] = dist.vb$r

   xy.dr = dr(y~x, method="save", nslices=5)
   v.est = xy.dr$evectors[ , c(1, 2)]
   dist.vb = dist.space(v.est, BB)
   er3[ii, 3] = dist.vb$r
}

colnames(er1) = c("FCN", "FMN", "SIR", "SAVE", "IHT", "y-pHd", "r-pHd")
colnames(er2) = c("FCN", "SIR", "SAVE")
colnames(er3) = c("FCN", "SIR", "SAVE")
postscript("examples.ps", paper="special", width=8, height=6, horizontal=F)
boxplot(as.data.frame(er1), main="Example: Rational Function")
boxplot(as.data.frame(er2), main="Example: Heteroscedastic Model")
boxplot(as.data.frame(er3), main="Example: Discrete Response")
dev.off()

dump(c("er1", "er2", "er3"), file="examples.out.Rdata")

dyn.unload("fourier-clib.dll")
date()
rm(list=ls())
quit("no")

